require 'securerandom'

module Bell::Ami
  class Caller
    attr_accessor :number, :ami_stream, :status, :call_id, :call_uniqueid, :options

    def initialize(number, ami_stream,options={})
      @number=number
      @ami_stream=ami_stream
      @ami_stream.register_handler(self)
      if @ami_stream.ami_stream.started?
        @status='connected'
      else
        @status='initialized'
      end
      @call_id=SecureRandom.uuid
      @call_uniqueid=nil

      @options={'Priority'=> '1', 'Timeout'=> '30000', 'MaxRetries'=>'0', 'RetryTime'=>'15', 'WaitTime'=> '45', }
      @options.merge!(options)
    end

    def handle_event(event)
      case event.name
      when 'FullyBooted'
        @status='ready'
      when 'Hangup'
        if event['Uniqueid']==@call_uniqueid
          if @status=='in_call'
            @status='succeeded'
          else
            case event['Cause']
            when '19'#User alerting, no answer
              @status='not_answered'
            when '17'#User busy
              @status='busy'
            when '16'#Normal Clearing
              @status='succeeded'
            else
              @status='not_available'
            end
          end
        end
      when 'NewCallerid'
        if event['CallerIDName']==@call_id
            @call_uniqueid=event['Uniqueid']
          end
      when 'Newexten'
        if event['Uniqueid']==@call_uniqueid
          if event['Application']=='Answer'
            @status='in_call'
          end
        end
      when 'DTMF'
        if event['Uniqueid']==@call_uniqueid
          if event['End']=='Yes'
            STDOUT.puts event['Digit']
          end
        end
      end
    end

    def handle_disconnected
      if @status='dialing'
        @status='errored'
      else
        @status='errored'
      end
    end

    def handle_connected
      @status='connected'
    end

    def dial(context, extention)
      @status='dialing'
      begin
         caller_id=%Q["#{@call_id}" <1000>]
        result=@ami_stream.ami_stream.send_action('Originate',@options.merge({"Channel"=>@number, 'Context'=> context, 'Exten'=> extention, 'Callerid': caller_id }))
      rescue RubyAMI::Error => e
        if @status=='dialing'
          @status='failed'
        end
      end
    end

    def terminate

    end

    def dialed?
      return @status=='dialing'
    end

    def finished?
      return @status=='errored' || @status=='failed' || @status=='succeeded' || @status=='not_answered' || @status=='busy' || @status=='not_available'
    end

    def ready?
      return @status=='ready'
    end

  end
end