require 'thor'
require 'bell/ami/caller'

module Bell::Ami
  class Cli < Thor
    def self.exit_on_failure?
      true
    end

    desc "call PHONENUMBER", "Call a phone number"
    long_desc <<-STARTDOC

    `call PHONENUMBER` will try to call the phone number.

    call command will use asterisk manager api to originate a call.
    Exit codes:
     0 : Successfull call
     1 : General error
     2 : Asterisk Manager connection error
     3 : Asterisk server error(maybe sip trunk is down)
     4 : Number did not answer
     5 : Number Busy
     6 : Number Not available/wrong
    
    If called number enter any key it will go to stdout.

    http://stackoverflow.com/a/12785204
    STARTDOC

    option :host, :required => true
    option :port
    option :username, required: true
    option :password, required: true
    option :context, required: true
    option :extension, required: true
    def call( phonenumber )
      logger=Logger.new(STDERR)
      my_ami=Bell::Ami::Stream.new(options[:host],options[:port] || 5038, options[:username], options[:password], logger)
      bell_caller=Bell::Ami::Caller.new(phonenumber,my_ami)

      logger.info "Connecting to #{options[:host]}"
      my_ami.async_run
      while !bell_caller.finished?
        if bell_caller.ready?
          logger.info "Calling #{phonenumber}"
          result=bell_caller.dial(options[:context],options[:extension])
        else
          # wait for call to finish
        end
        sleep 1
      end

      #terminal stream
      my_ami.terminate

      case bell_caller.status
      when 'succeeded'
        exit 0
      when 'errored'
        exit 2
      when 'failed'
        exit 3
      when 'not_answered'
        exit 4
      when 'busy'
        exit 5
      when 'not_available'
        exit 6
      end
      exit 1
    end

  end
end
