require 'ruby_ami'

module Bell
  module Ami
    class Stream
      attr_accessor :host, :port, :user, :password, :ami_stream, :logger, :event_handlers

      def initialize(host,port,user,password, logger)
        @host=host
        @port=port
        @user=user
        @password=password
        @logger=logger
        @ami_stream = RubyAMI::Stream.new @host, @port, @user, @password, lambda{|e| self.handle_event(e) }, @logger, 10
        @event_handlers=[]
      end

      def login
        @ami_stream.login
      end

      def handle_event(event)
        if event.class.name=='RubyAMI::Stream::Disconnected'
          #handle disconnection
          event_handlers.each do |event_handler|
            if event_handler.class.method_defined? :handle_disconnected
              event_handler.handle_disconnected
            end
          end
        elsif event.class.name=='RubyAMI::Stream::Connected'
          event_handlers.each do |event_handler|
            if event_handler.class.method_defined? :handle_connected
              event_handler.handle_connected
            end
          end
        elsif event.class.name=='RubyAMI::Event'
          event_handlers.each do |event_handler|
            if event_handler.class.method_defined? :handle_event
              event_handler.handle_event(event)
            end
          end
        end
      end

      def async_run
        @ami_stream.async.run
      end

      def status
        @ami_stream.send_action('Status')
      end

      def queue_status(queue_name)
        @ami_stream.send_action('Queuestatus',{"Queue"=>queue_name })
      end

      def register_handler(handler)
        @event_handlers << handler
      end

      def unregister_handler(handler)
        @event_handlers.delete(handler)
      end

      def terminate
        if @ami_stream.alive? && @ami_stream.started?
          @ami_stream.send_action('LOGOFF')
        end
      end
    end
  end
end
